#!/usr/bin/perl -w

use strict;

my $inputfile = '-';
my $ignore = 20;
my $tab;
my $inputhandle;

&main();

sub main {
	&getOptions();
	&readInputandOutput();
}

sub getOptions {
	# get arguments
	for (my $i = 0; $i < scalar(@ARGV); $i ++) {
		if ($ARGV[$i] =~ /^-+ignore$/i && $ARGV[($i + 1)] =~ /^\d+$/) {
			$ignore = $ARGV[($i + 1)];
			$i ++;
		}
		elsif ($ARGV[$i] =~ /^-+tab$/i) {
			$tab = 1;
		}
		elsif ($ARGV[$i] =~ /^-+(?:V|version)$/i) {
			print(STDERR "nucmer2ovl ver. 0.1\n");
			exit(0);
		}
		elsif ($ARGV[$i] =~ /^-+(?:h|help)$/i) {
			&helpMessage();
		}
		elsif (-e $ARGV[$i]) {
			$inputfile = $ARGV[$i];
		}
		else {
			&errorMessage(__LINE__, "\"$ARGV[$i]\" is invalid argument.");
		}
	}
}

sub readInputandOutput {
	my %checkdup;
	unless (open($inputhandle, "< $inputfile")) {
		&errorMessage(__LINE__, "Cannot read \"$inputfile\".");
	}
	# parse input
	while (<$inputhandle>) {
		my @rows = split(/\s+/, $_);
		my $nrows = scalar(@rows);
		if ($nrows > 12 && $rows[0] =~ /^\d+$/) {
			if ($rows[-1] =~ /\[/) {
				pop @rows;
				$nrows --;
			}
			my ($a_id, $a_start, $a_end, $a_len) = ($rows[($nrows - 2)], $rows[0], $rows[1], $rows[11]);
			my ($b_id, $b_start, $b_end, $b_len) = ($rows[($nrows - 1)], $rows[3], $rows[4], $rows[12]);
			my ($dir, $switched, $a_hang, $b_hang);
			if ($a_id eq $b_id) {
				next;
			}
			if ($checkdup{$a_id}{$b_id} || $checkdup{$b_id}{$a_id}) {
				next;
			}
			# SAME DIR
			if ($b_start < $b_end) {
				if ($a_start < $b_start) {
					($a_id, $a_start, $a_end, $a_len, $b_id, $b_start, $b_end, $b_len) = ($b_id, $b_start, $b_end, $b_len, $a_id, $a_start, $a_end, $a_len);
					$switched = 1;
				}
				# CONTAINS
				if ($b_start < $ignore && ($b_len - $b_end) < $ignore) {
					$a_hang = $a_start - 1;
					$b_hang = $a_end - $a_len;
					$dir = "N";
				}
				# NORMAL
				elsif (($a_len - $a_end) < $ignore && $b_start < $ignore) {
					$a_hang = $a_start - 1;
					$b_hang = $b_len - $b_end;
					$dir = "N";
				}
			}
			# OPPOSITE DIR
			else {
				($b_start, $b_end) = ($b_end, $b_start);
				if ($a_start < $b_start) {
					($a_id, $a_start, $a_end, $a_len, $b_id, $b_start, $b_end, $b_len) = ($b_id, $b_start, $b_end, $b_len, $a_id, $a_start, $a_end, $a_len);
					$switched = 1;
				}
				# CONTAINS
				if ($b_start < $ignore && ($b_len - $b_end) < $ignore) {
					$a_hang = $a_start - 1;
					$b_hang = $a_end - $a_len;
					$dir = "I";
				}
				# OUTIE
				elsif ($a_start < $ignore && $b_start < $ignore) {
					$a_hang = $b_end - $b_len;
					$b_hang = $a_end - $a_len;
					$dir = "I";
				}
				# INNIE
				elsif (($a_len - $a_end) < $ignore && ($b_len - $b_end) < $ignore) {
					$a_hang = $a_start - 1;
					$b_hang = $b_start - 1;
					$dir = "I";
				}
			}
			if ($dir) {
				$checkdup{$a_id}{$b_id} = 1;
				if ($switched) {
					($a_id, $b_id) = ($b_id, $a_id);
					if ($dir eq "N") {
						($a_hang, $b_hang) = ((-1 * $a_hang), (-1 * $b_hang));
					}
					else {
						($a_hang, $b_hang) = ($b_hang, $a_hang);
					}
				}
				if ($tab) {
					print(join("\t", ($a_id, $b_id, $dir, $a_hang, $b_hang, $rows[6], $rows[9])) . "\n");
				}
				else {
					print("{OVL\nadj:$dir\nrds:$a_id,$b_id\nscr:0\nahg:$a_hang\nbhg:$b_hang\n}\n");
				}
			}
		}
	}
}

sub errorMessage {
	my $lineno = shift(@_);
	my $message = shift(@_);
	print(STDERR "ERROR!: line $lineno\n$message\n");
	print(STDERR "If you want to read help message, run this script without options.\n");
	exit(1);
}

sub helpMessage {
	print(STDERR <<"_END");
Usage
=====
nucmer2ovl options input > output
nucmer2ovl options < input > output
cat input | nucmer2ovl options > output

Command line options
====================
-ignore INTEGER
  Maximum length of the end sequence unaligned (Default: 20 bp)

-tab
  Output format (Default: AMOS message)

-h|help
  Print this help and exit;

-V|version
  Print the version and exit;

Acceptable input file formats
=============================
show-coords output
_END
	exit;
}
