#!/usr/bin/perl -w

use strict;

my $inputfile = '-';
my $ignore = 20;
my $all;
my $loose;
my $noid;
my $inputhandle;

&main();

sub main {
	&getOptions();
	&readInputandOutput();
}

sub getOptions {
	# get arguments
	for (my $i = 0; $i < scalar(@ARGV); $i ++) {
		if ($ARGV[$i] =~ /^-+ignore$/i && $ARGV[($i + 1)] =~ /^\d+$/) {
			$ignore = $ARGV[($i + 1)];
			$i ++;
		}
		elsif ($ARGV[$i] =~ /^-+all$/i) {
			$all = 1;
		}
		elsif ($ARGV[$i] =~ /^-+loose$/i) {
			$loose = 1;
		}
		elsif ($ARGV[$i] =~ /^-+noid$/i) {
			$noid = 1;
		}
		elsif ($ARGV[$i] =~ /^-+(?:V|version)$/i) {
			print(STDERR "nucmerAnnotate ver. 0.1\n");
			exit(0);
		}
		elsif ($ARGV[$i] =~ /^-+(?:h|help)$/i) {
			&helpMessage();
		}
		elsif (-e $ARGV[$i]) {
			$inputfile = $ARGV[$i];
		}
		else {
			&errorMessage(__LINE__, "\"$ARGV[$i]\" is invalid argument.");
		}
	}
}

sub readInputandOutput {
	unless (open($inputhandle, "< $inputfile")) {
		&errorMessage(__LINE__, "Cannot read \"$inputfile\".");
	}
	# parse input
	while (<$inputhandle>) {
		s/\r?\n?$//;
		my @rows = split(/\s+/, $_);
		my $nrows = scalar(@rows);
		if ($nrows > 12 && $rows[0] =~ /^\d+$/) {
			if ($rows[-1] =~ /\[/) {
				pop(@rows);
				$nrows --;
			}
			$rows[$nrows] = "";
			if ($noid && $rows[($nrows - 2)] eq $rows[($nrows - 1)] && $rows[11] == $rows[12]) {
				next;
			}
			my $rev;
			if ($rows[3] > $rows[4]) {
				($rows[3], $rows[4], $rev) = ($rows[4], $rows[3], 1);
			}
			if ($rows[0] <= $ignore && $rows[11] - $rows[1] <= $ignore && $rows[3] <= $ignore && $rows[12] - $rows[4] <= $ignore) {
				$rows[$nrows] = "[IDENTITY]";
			}
			elsif ($rows[0] <= $ignore && $rows[11] - $rows[1] <= $ignore) {
				$rows[$nrows] = "[CONTAINED]";
			}
			elsif ($rows[3] <= $ignore && $rows[12] - $rows[4] <= $ignore) {
				$rows[$nrows] = "[CONTAINS]";
			}
			elsif ($loose && $rows[3] <= $ignore) {
				$rows[$nrows] = "[BEGIN]";
			}
			elsif ($loose && $rows[12] - $rows[4] <= $ignore) {
				$rows[$nrows] = "[END]";
			}
			elsif (($rows[3] <= $ignore || $rows[12] - $rows[4] <= $ignore) && $rows[0] <= $ignore) {
				$rows[$nrows] = "[BEGIN]";
			}
			elsif (($rows[3] <= $ignore || $rows[12] - $rows[4] <= $ignore) && $rows[11] - $rows[1] <= $ignore) {
				$rows[$nrows] = "[END]";
			}
			if ($rev) {
				($rows[3], $rows[4]) = ($rows[4], $rows[3]);
			}
			if ($rows[$nrows] || $all) {
				print(join("\t", @rows) . "\n");
			}
		}
	}
	close($inputhandle);
}

sub errorMessage {
	my $lineno = shift(@_);
	my $message = shift(@_);
	print(STDERR "ERROR!: line $lineno\n$message\n");
	print(STDERR "If you want to read help message, run this script without options.\n");
	exit(1);
}

sub helpMessage {
	print(STDERR <<"_END");
Usage
=====
nucmerAnnotate options input > output
nucmerAnnotate options < input > output
cat input | nucmerAnnotate options > output

Command line options
====================
-ignore INTEGER
  Maximum length of the end sequence unaligned (Default: 20 bp)

-all
  Display all alignments (Default: only the annotated ones)

-loose
  Looser annotation

-noid
  Filter out identity alignments (sequences with the same id)

-h|help
  Print this help and exit;

-V|version
  Print the version and exit;

Acceptable input file formats
=============================
show-coords output
_END
	exit;
}
